/* smurf.h of iplog v1.3 -- eric@ojnk.nu */
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <netdb.h>

#define HDR_SIZE sizeof(struct ip)

#define THRESH_PPS	25			/* packets per second */
#define THRESH_KBPS	20			/* kilobytes per second */

time_t last;
float kbps = 0.0;
int count = 0, smurf_mode = 0, pps = 0;
unsigned long int *ips;

int comp(const void *a, const void *b)
{
	return *(int *)a - *(int *)b;
}

/* based very heavily on buglord's SmurfLog, but modified to work in iplog */
int smurf(struct packet pkt)
{
	time_t current;
	u_char *p;
	unsigned long int the_ip;

	if (smurf_mode) {
		if (time(NULL) >= (last + 15)) {
			free(ips);
			smurf_mode = 0;
			return (1);
		} else {
			p = (unsigned char *)&pkt.ip.ip_src.s_addr;
			p[3] = 0;

			the_ip = pkt.ip.ip_src.s_addr;

			if (bsearch(&the_ip, ips, count, sizeof(*ips), comp) == NULL) {
				syslog(LEVEL, "smurf attack detected from %s/24 (%d bytes)", inet_ntoa(pkt.ip.ip_src), htons(pkt.ip.ip_len) - HDR_SIZE);

				if ((ips = realloc(ips, count + 1)) == NULL)
					fatal("smurf prot realloc: %m");

				ips[count++] = the_ip;
				qsort(ips, count, sizeof(*ips), comp);
			}
			time(&last);
			return (0);
		}
	} else {
		kbps += ntohs(pkt.ip.ip_len) / 1024.0;
		pps++;
		time(&current);

		if ((current - last) >= 1 || (current < last)) {
			pps = 0;
			kbps = 0.0;
			time(&last);
			return (1);
		} else if (pps >= THRESH_PPS && kbps >= THRESH_KBPS) {
			smurf_mode = 1;
			syslog(LEVEL, "threshold met (%ld pkt/s, %4.02fkb/s)", pps, kbps);
			time(&last);
			return (0);
		}
		time(&last);
		return (1);
	}
}
